/**
 * @(#)City.java  1.0  2 April 2007
 *
 * @author Viera K. Proulx
 */

import interfaces.*;
import edu.neu.ccs.*;	    // to get XInt

import java.util.*;         // to get Scanner
import java.text.*;         // to get ParseException

/** A simple class to use for learning about classes and objects. */
class City implements ISame<City>, Stringable{

  /*-------------------------------------------------------------------------
     Member data 
   *-----------------------------------------------------------------------*/

  /** The zip code for this city. */
  public int zip;

  /** The name of this city. */
  public String name;

  /** The state for this city */
  public String state;

  /** The longitude for this city */
  public double longitude;

  /** The latitude for this city */
  public double latitude;

  /** Decimal format to print leading zeros in zip code */
  public DecimalFormat zipFormat = new DecimalFormat("00000");

  /*-------------------------------------------------------------------------
     Constructor
   *-----------------------------------------------------------------------*/

  /** The full constructor */
  public City (int zip, String name, String state, 
      double longitude, double latitude){
    this.zip   = zip;
    this.name  = name;
    this.state = state;
    this.longitude = longitude;
    this.latitude  = latitude;
  }    
  /** The copy constructor */
  public City(City aCity){
    this(aCity.zip, aCity.name, aCity.state, aCity.longitude, aCity.latitude);
  }

  /** The constructor that allows zip code input as a <code>String</code> */
  public City (String zip, String name, String state, 
      double longitude, double latitude){
    this.zip = this.decimal(zip);
    this.name  = name;
    this.state = state;
    this.longitude = longitude;
    this.latitude  = latitude;
  }

  /** The default constructor */
  public City(){
    this.zip   = 48170;
    this.name  = "Plymouth";
    this.state = "MI";
    this.longitude  = 83.479946;
    this.latitude = 42.368820;
  }

  /*-------------------------------------------------------------------------
     Methods 
   *-----------------------------------------------------------------------*/
  /** 
   * Determine whether this city is in the given state 
   * @param aState the given state
   */
  boolean isInState(String aState){
    return this.state.equals(aState);
  }


  /*-------------------------------------------------------------------------
   Implementing the ISame interface
   *-----------------------------------------------------------------------*/
  /** determine whether this city is the same as the given object */
  public boolean same(City that){
    return
    this.zip == that.zip &&
    this.name.equals(that.name) &&
    this.state.equals(that.state) &&
    this.sameDouble(this.longitude, that.longitude, 0.000001) &&
    this.sameDouble(this.latitude, that.latitude, 0.000001);
  }

  /** helper method to compare doubles */
  protected boolean sameDouble(double d1, double d2, double limit){
    return Math.abs(d1 - d2) < limit;
  }
  
  /** override the <code>equals</code> method */
  // public boolean equals(Object obj){ 
  // ...
  
  /** override the <code>hashCode</code> method */
  //  public int hashCode(){
  // ............................
  

  /*-------------------------------------------------------------------------
     Implementing theStringable interface
   *-----------------------------------------------------------------------*/
  /** Represent city data as a string (for file output) */
  public String toStringData(){
    String s = "" + this.zipFormat.format(this.zip) + ", " +
    this.name  + ", " +
    this.state + ", " +
    this.longitude + ", " +
    this.latitude  + "\n";
    return s;
  }

  /** Initialize city fields from a String */
  public void fromStringData(String s) throws java.text.ParseException{

    try{
      /** set up the scanner to extract city data */

      Scanner st = new Scanner(s).useDelimiter("[,]");
      this.zip       = decimal(st.next());
      this.name      = st.next().trim();
      this.state     = st.next().trim();
      this.longitude = doubleValue(st.next());
      this.latitude  = doubleValue(st.next());
    }
    catch(NoSuchElementException e){
      System.out.println(e.getMessage());
      throw new java.text.ParseException("Tokenizer Error", 0);
    }
  }

  /** Helper: convert a String to int - if possible */
  protected int intValue(String s) throws ParseException{

    return (new XInt(s)).intValue();
  }    

  /** convert a String to double - if possible */
  protected double doubleValue(String s) throws ParseException{
        return (new XDouble(s)).doubleValue();
  }
  
  /*-------------------------------------------------------------------------
     Console print
   *-----------------------------------------------------------------------*/
  /** Represent  city data as a String for printed display */
  public String toString(){
    return ("new " + getClass() + "(" + 
        this.zipFormat.format(this.zip)   + ",   " + 
        this.name  + ",  " + 
        this.state + ", " + 
        this.longitude + ", " + 
        this.latitude + ")\n");
  }

  /** Helper: converts zip code String to integer */
  public int decimal(String s){

    char[] c = s.toCharArray();
    int n = 0;
    try {
      for (int i = 0; i < 5; i++){
        n = 10 * n + intValue("" + c[i]);
      }
    }
    catch(ParseException e){
      System.out.println("Converting zip to int " + e);
    }
    return n;
  }

}
