/**
 *  
 * Functional (External) Iterator Pattern:
 * iterator for a list of any Stringable objects.
 * 
 *  Warning: User supplies an instance of Stringable
 *  on each iteration this instance is mutated to represent 
 *  new data. 
 *  User must create a persistent copy using an appropriate 
 *  copy constructor.
 */

import edu.neu.ccs.*;
import edu.neu.ccs.gui.*;
import edu.neu.ccs.util.*;
import interfaces.*;

import java.awt.*;
import java.io.*;
import javax.swing.*;
import java.util.*;

/** An input iterator for Stringable object from file input. */
class InFileBufferedTraversal implements Traversal<Stringable>{
  
  /*-------------------------------------------------------- 
   Member data
   *------------------------------------------------------*/
  
  /** the input reader */
  protected BufferedReader buffer;
  
  /** one line of input at a time */
  protected String line;
  
  /** a Stringable object to hold values */
  protected Stringable s;
  
  /** determines whether the general dialog has been closed */
  protected boolean closed = true;
  
  /** determines whether new Stringable data has been submitted */
  protected boolean submitted = false;
  
  /*-------------------------------------------------------- 
   Constructor
   *------------------------------------------------------*/    
  public InFileBufferedTraversal(Stringable s) {
    this.s = s;
    
    /** build a file chooser 
     and have the user choose a file, 
     quitting this operation 
     if the user cancelled the choice */
    JFileChooser chooser = new JFileChooser(".");
    
    /** set the file extension to be .txt */
    chooser.setFileFilter(new FileView.ExtensionFileFilter("txt"));
    
    closed = false;
    
    /** see if file was selected - quit if user canceled */
    if (chooser.showOpenDialog(null) != JFileChooser.APPROVE_OPTION){
      closed = true;
      return;
    }
    try{
      buffer = new BufferedReader(
          new FileReader(chooser.getSelectedFile()));
    }
    catch(FileNotFoundException e){
      System.out.println("File not found exception: " + e);
      closed = true;        
    }
    
    
    /** read one line of input from the selected file 
     *  and extract values for the first city */
    getnext();
  }
  /*--------------------------------------------------------
   Methods to implement the Traversal interface
   *------------------------------------------------------*/
  
  /** process next item, if available */
  public void getnext() {
    
    if (!closed)
      try{
        if (buffer.ready()){
          
          line = new String(buffer.readLine());
          if ((line == null)||(line == "")){     
            buffer.close();
            closed = true;
          }
          else{ 
            read(line);
          }            
        }
        else{
          buffer.close();
          closed = true;
        }
      }
    catch(Exception e) {
      System.out.println("Error in reading line exception: " + e);
      closed = true;
    }
    
  }
  
  /** advance to the next element in the dataset */
  public Traversal<Stringable> getRest(){
    getnext();
    return this;
  }
  
  /** return the most recent submission - if available */
  public Stringable getFirst() {
    
    if (!closed)
      return s;
    else{
      closed = true;
      return null;
    }
  }
  
  /** verify that new submission is available */
  public boolean isEmpty() {
    return closed; 
  }
  
  /** extract next Stringable object from the input string */
  public void read(String str){
    
    try {
      s.fromStringData(str);
    }
    catch(java.text.ParseException e){
      System.out.println(e.getMessage() + 
          " parse exception in line\n" + 
          line);
      s = null;
      closed = true;
    }
    
  }
  
}