/* ------------------------------------------------------------------
   Functional (External) Iterator Pattern:
        iterator for a list of cities 
*/

import edu.neu.ccs.*;
import edu.neu.ccs.gui.*;
import edu.neu.ccs.util.*;
import interfaces.*;

import java.awt.*;
import java.io.*;
import javax.swing.*;
import java.util.*;

/** An input iterator for Stringable object from file input. */
class OutFileTraversal implements Traversal<Stringable>{
    
    /*-------------------------------------------------------- 
     Member data
     *------------------------------------------------------*/
    
    /** the list of T objects to output */
    protected ArrayList<Stringable> objects;
    
    /** the loop counter */
    int i = 0;
    
    /** true if the file is closed */
    boolean closed = false;
    
    /** a String object to hold file contents */
    protected String s = "";

    /** file chooser - to select the file from */
    protected JFileChooser chooser;
        
    /*-------------------------------------------------------- 
     Constructor
     *------------------------------------------------------*/    
    /** the constructor
     * 
     * @param theObjects data to be saved to a file stored in an Array 
     */
    public OutFileTraversal(ArrayList<Stringable> theObjects) {
        
        this.objects = theObjects;
                     
        /** build a file chooser 
            and have the user choose a file, 
            quitting this operation 
            if the user cancelled the choice */
        chooser = new JFileChooser(".");
        
        /** set the file extension to be .txt */
        chooser.setFileFilter(new FileView.ExtensionFileFilter("txt"));
        
        /** see if file was selected - quit if user canceled */
        if (chooser.showSaveDialog(null) != JFileChooser.APPROVE_OPTION){
            i = objects.size();
            closed = true;
            return;
        }
        else
        	closed = false;
        
    }
    /*--------------------------------------------------------
     Methods to implement the Traversal interface
     *------------------------------------------------------*/

    /** process next item, if available */
    public Traversal<Stringable> getRest() {

        String temp = s;
        if (hasMore()){
            s = new String(temp + objects.get(i).toStringData());
            i++;
        }
        return this;
    }

    /** return the most recent object processed - if available */
    public Stringable getFirst() {
           
        if (hasMore())
            return objects.get(i);
        return null;
    }
    
    /** verify that new object is available to add to the file*/
    public boolean isEmpty() {
      return !this.hasMore();
    }
 
    /** verify that new object is available to add to the file*/
    public boolean hasMore() {
        
        /** more objects available */
        if ((i < objects.size()) && (!closed))
            return true;

        if (!closed){
	        /** no more items to add to the file */
	        /** OK to overwrite existing file    */
	        try {
	            FileUtilities.writeFile(
	            	chooser.getSelectedFile(),s,true);
	            closed = true;
	        }
	        catch (IOException e){
	            System.out.println(e.getMessage() + " Error saving:\n" + s);
	        	closed = true;
	        }
        }
        return false; 
    }    
}